@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -698,8 +699,7 @@ USA.
     (lambda (port)
       (write-string "#| -*-Scheme-*-\n\n" port)
       (write-copyright+license pathname port)
-      (newline port)
-      (write-string "|#\n\n" port)
+      (write-string "\n|#\n\n" port)
       (write-string ";;;; " port)
       (write-string title port)
       (write-string "\n\n" port)
@@ -763,8 +763,7 @@ USA.
 			    "_H")))
 	(write-string "/* -*-C-*-\n\n" port)
 	(write-copyright+license pathname port)
-	(newline port)
-	(write-string "*/\n\n" port)
+	(write-string "\n*/\n\n" port)
 	(write-string "/* " port)
 	(write-string title port)
 	(write-string " */\n\n" port)
@@ -854,10 +853,10 @@ USA.
   (write-string "DO NOT EDIT: this file was generated by a program." port)
   (newline port)
   (newline port)
-  (write-mit-scheme-copyright port)
+  (write-mit-scheme-copyright port "" "(C)" #t)
   (newline port)
   (newline port)
-  (write-mit-scheme-license port)
+  (write-mit-scheme-license port "" #t)
   (newline port))
 
 (define (name->c-string name upcase?)
@@ -1000,16 +999,27 @@ USA.
 
 (define (rt-defn-encoder-constructor defn)
   `(LAMBDA (INSTANCE WRITE-BYTE)
-   ,@(if (null? (defn-coding defn))
-       '(INSTANCE WRITE-BYTE UNSPECIFIC)
-       (map (lambda (item)
-	      (let ((pval `(RT-INSTANCE-PVAL ',(pvar-name item) INSTANCE))
-		    (pvt (lookup-pvar-type (pvar-type item))))
-		(if pvt
-		    `(,(pvt-encoder pvt) ,pval WRITE-BYTE)
-		    `(LET ((PVAL ,pval))
-		       ((RT-INSTANCE-ENCODER PVAL) PVAL WRITE-BYTE)))))
-	    (defn-coding defn)))))
+     ,@(if (null? (defn-coding defn))
+	   (let ((code (defn-code defn)))
+	     (if code
+		 `(INSTANCE
+		   (WRITE-BYTE ,code))
+		 (error "Nothing to encode:" defn)))
+	   `(,@(let ((code (defn-code defn)))
+		 (if code
+		     `((WRITE-BYTE ,code))
+		     ;; The datum-* pseudo-instructions have no (op)code.
+		     '()))
+	     ,@(map (lambda (item)
+		      (let ((pval `(RT-INSTANCE-PVAL
+				    ',(pvar-name item) INSTANCE))
+			    (pvt (lookup-pvar-type (pvar-type item))))
+			(if pvt
+			    `(,(pvt-encoder pvt) ,pval WRITE-BYTE)
+			    `(LET ((PVAL ,pval))
+				  ((RT-INSTANCE-ENCODER PVAL)
+				   PVAL WRITE-BYTE)))))
+		    (defn-coding defn))))))
 
 (define (rt-defn-decoder-constructor defn)
   (let ((pvars (defn-pvars defn)))
@@ -1019,12 +1029,10 @@ USA.
 	     (let ((pvt (lookup-pvar-type (pvar-type pv))))
 	       (if pvt
 		   `(,(pvt-decoder pvt) READ-BYTE)
-		   `(DECODE-RT-CODING-TYPE ',(pvar-type pv)
-					   READ-BYTE
-					   CODING-TYPES))))))
+		   `(DECODE-RT-CODING-TYPE ',(pvar-type pv) READ-BYTE))))))
       `(LAMBDA (READ-BYTE)
       ,@(cond((fix:= n-pvars 0)
-	      `(READ-BYTE CODING-TYPES '()))
+	      `(READ-BYTE '()))
 	     ((fix:= n-pvars 1)
 	      `((LIST ,(body (car pvars)))))
 	     (else
@@ -1042,4 +1050,11 @@ USA.
 	(error:bad-range-argument pv 'PVAR-INDEX))
     (if (eq? (car pvars) pv)
 	index
-	(loop (cdr pvars) (fix:+ index 1)))))
\ No newline at end of file
+	(loop (cdr pvars) (fix:+ index 1)))))
+
+;;; This hides the rule-matcher forms in assembler-runtime.
+(define-syntax RULE-MATCHER 
+  (rsc-macro-transformer
+   (lambda (form environment)
+     form environment
+     #f)))
\ No newline at end of file
